'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _provider = require('./provider.js');

var _provider2 = _interopRequireDefault(_provider);

var _testauth = require('./testauth.js');

var _testauth2 = _interopRequireDefault(_testauth);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var WhatAuth = function () {

  // Merge the given config with defaults and initialize the providers.
  function WhatAuth(config) {
    (0, _classCallCheck3.default)(this, WhatAuth);

    var all = (0, _assign2.default)({}, defaults.all, config && config.all);
    this.providers = {};
    for (var key in config) {
      if (key === 'all') {
        continue;
      }
      var providerConfig = (0, _assign2.default)({}, all, defaults[key], config[key]);
      this.providers[key] = new _provider2.default(providerConfig);
    }
  }

  // Find the appropriate OAuth provider and use it.


  (0, _createClass3.default)(WhatAuth, [{
    key: 'fetch',
    value: function fetch(query) {
      if (!query) {
        throw new Error('Missing query.');
      }
      var provider = this.providers[query.provider] || this.providers[query.provider.split('-')[0]];
      if (!provider) {
        throw new Error('Unknown provider: ' + query.provider);
      }
      return provider.fetch(query);
    }
  }]);
  return WhatAuth;
}();

exports.default = WhatAuth;


var defaults = {
  all: {
    imageSize: 200,
    baseSite: '',
    process: function process(data) {
      return data;
    }
  },
  testauth: {
    OAuth: _testauth2.default
  },
  facebook: {
    baseSite: "https://graph.facebook.com",
    fields: "name,email",
    profileBase: "https://graph.facebook.com/v2.5/me?fields=",
    process: function process(data) {
      return {
        ident: "facebook:" + data.id,
        name: data.name,
        image: "https://graph.facebook.com/" + data.id + "/picture" + "?width=" + this.imageSize + "&height=" + this.imageSize,
        email: data.email,
        data: data
      };
    }
  },
  google: {
    accessTokenPath: "https://www.googleapis.com/oauth2/v4/token",
    //accessTokenPath: "https://accounts.google.com/o/oauth2/token",
    params: { grant_type: "authorization_code" },
    profileUrl: "https://www.googleapis.com/plus/v1/people/me",
    process: function process(data) {
      var email = null;
      try {
        email = data.emails[0].value;
      } catch (e) {
        null;
      }
      var image = null;
      try {
        image = data.image.url.split('?')[0] + '?sz=' + this.imageSize;
      } catch (e) {
        null;
      }
      return {
        ident: 'google:' + data.id,
        name: data.displayName,
        image: image,
        email: email,
        data: data
      };
    }
  },
  github: {
    baseSite: "https://github.com/login",
    //accessTokenPath: "https://github.com/login/oauth/access_token",
    profileUrl: "https://api.github.com/user",
    emailUrl: "https://api.github.com/user/emails",
    process: function process(data) {
      var email = data.email;
      if (!email) {
        var _iteratorNormalCompletion = true;
        var _didIteratorError = false;
        var _iteratorError = undefined;

        try {
          for (var _iterator = (0, _getIterator3.default)(data.emails), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var obj = _step.value;

            if (obj.primary) {
              email = obj.email;break;
            }
          }
        } catch (err) {
          _didIteratorError = true;
          _iteratorError = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion && _iterator.return) {
              _iterator.return();
            }
          } finally {
            if (_didIteratorError) {
              throw _iteratorError;
            }
          }
        }
      }
      return {
        ident: 'github:' + data.id,
        name: data.name || data.login,
        image: data.avatar_url,
        email: email,
        data: data
      };
    }
  }
  // TODO Bitbucket
  // TODO StackOverflow
  // TODO Odnoklassniki
};